import thinlet.Thinlet;
import thinlet.FrameLauncher;
import java.lang.*;
import java.sql.*; 
import java.awt.*;
import java.applet.Applet;
import java.util.Properties;
import java.math.*;

//
// Your Momma Loves Drama 
// Apache Derby - Off to the Races
// Copyright (c) 2005 - George Baklarz, Paul Zikopoulos, Dan Scott
//

public class YMLD extends Thinlet {

    //
    // String constants for the location of the panels used in YMLD. The panels
    // are loaded in by the Thinlet package and need to be available in either the
    // directory /panels, or in the jar file that the program is zipped into. 
    //
    static final String 
        ymldMain          = "panels/YMLD-Main.xml",
        ymldConnect       = "panels/YMLD-Connect.xml",
        ymldMessage       = "panels/YMLD-Message.xml",
        ymldProductions   = "panels/YMLD-Productions.xml",
        ymldPricePlans    = "panels/YMLD-PricePlans.xml",
        ymldShowSeats     = "panels/YMLD-Seats.xml",
        ymldProdSelection = "panels/YMLD-ProdSelection.xml",
        ymldSeatSelection = "panels/YMLD-SeatSelection.xml",
        ymldShowTickets   = "panels/YMLD-ShowTickets.xml",
        ymldTransactions  = "panels/YMLD-Transactions.xml",
        ymldUndo          = "panels/YMLD-Undo.xml",
        ymldInfo          = "panels/YMLD-Info.xml"
    ;

    //
    // monthDays  - The days found in each month of the year.
    // monthDow   - The "day" of the week that the first of th/e month starts on
    // monthNames - The name of the months.
    // minMonth   - The first month that a production starts in
    // maxMonth   - The last month that a production stops in
    // Notes:      
    // The arrays all have values starting at position 1 rather than 0.
    // This simplifies the date logic since we can use the absolute day or
    // month number.
    //
    static final int [] monthDays = {0,31,28,31,30,31,30,31,31,30,31,30,31};
    static final int [] monthDoW  = {0,6,2,2,5,0,3,5,1,4,6,2,4};
    static final String [] monthNames = {"","January","February","March","April",
                                     "May","June","July","August","September",
	 								 "October","November","December"};
    static int 	minMonth = 1;
    static int	maxMonth = 12;
  
    //
    // Panel objects used for tracking which panels are currently open. These
    // objects need to be used to close multiple panels when a ticket has been
    // purchased. When a customer is buying a ticket, they will go through 3 steps:
    // 1) Select a production
    // 2) Select a date from the list of dates available
    // 3) Select one or more seats to purchase	
    // 4) Review the seats that were selected
    // 5) Confirm the order
    // At any stage, the customer can go back to a previous screen to modify their
    // selection. When they finally complete their transaction, the program must
    // close all of the intermediate panels. The easiest way to accomplish this
    // is to set up panel objects which can all be "closed" when the transaction
    // completes.
    //
    static Object 
	    panelSeatSelection,                            // Seat Selection
        panelProdSelection,		 				  	   // Production Selection
        panelConfirmSelection					 	   // Ticket Confirmation
    ;
	   	 
    //
    // Various class variables used for tracking the performance that is selected
    // by a customer.
    //
    static String selectedProdName;                    // Production Name
    static int    selectedProdNo;		    		   // Production Number
    static String selectedProdDate;         		   // Production Date
    static int    selectedProdMonth;				   // Production Month
    static int    selectedProdDay;					   // Production Day
    static int    selectedPerfNo;					   // Performance number
    static int     [] selectedSeats = new int[21];     // Which seats do you want?
    static String  [] productionNames = new String[10];// Names of all of the productions
    static boolean [] prodDays = new boolean[32]; 	   // Days on which a production is scheduled
    static int     [] prodPerf = new int[32]; 		   // Performance number for a particular day
    static String selectedProduction;				   // Name of the production selected
    static int    startMonth;						   // The month that the production starts in
    static int    endMonth;						       // The ending month for a production
    static int    totalCost = 0;					   // Total cost of all tickets purchased
	
    //
    // Class variables for deleting an existing transaction
    //
    static String deleteDate;						   // Date of the transaction being deletd
    static String deletePerfNo;					       // Performance number of the tx being deleted
    static String deleteSeatNo;					       // Seat number of the tx being deleted
    static String deleteTxNo;						   // Transaction number of the tx being deleted
    static Object currentRow;						   // Current row selected in the transaction log
	
	//
	// Connection handle (conn) for all subsequent SQL statements.
	//
    static Connection conn = null;				       // Connection handle to the YMLD database	
	
	//
	// Main YMLD panel and miscellaneous logic
    //
    // The main section of code contains three routines:
    // main()               - main class of the program - invokes GUI
    // YMLD()               - opens up the main panel and the connection screen
    // connectYMLD()        - issues the connect statements to the database
    // pClosePanel()        - generic routine for closing a panel
    // exitYMLD()           - close all open connections and exit the program
    // info()               - Information panel about the YMLD program
    // showmessage()        - display an error message on the screen
    // getSQLError()        - gets the SQL error message and displays it
    //
    
 
	//
	// The main program will call YMLD to initialize the first panel in the
	// YMLD program. It will display the main screen along with the connection
	// dialog since a user must sign in to use this.
	//
	public static void main(String[] args) throws Exception {
        new FrameLauncher("Your Momma Loves Drama", new YMLD(), 580, 375);
	}
	
	//
	// The YMLD method displays the main YMLD panel, along with the connection
	// panel. The Connection panel is given focus over the main panel since 
	// a valid connection must exist to the database before the program
	// can run.
	// 
	public YMLD() throws Exception {
		Object panel = parse(ymldMain);				   // Show main YMLD panel
		add(panel);
		panel = parse(ymldConnect);					   // Also show connect panel
		add(panel);
		requestFocus(find(panel,"bConnect"));		   // Focus on Connect button
	}
		
	//
	// connectYMLD() establishes a connection to the YMLD Database.
	// The connectYMLD method is used to create a connection to the YMLD
	// database. It loads the appropriate network driver and then
	// connects using the userid and password supplied in the functions. Note
	// that this version of YMLD uses the embedded driver, not the network
	// driver. This can be easily changed by updating the value for 
	// DerbyDriver in this method.
	// panel  - panel object to be closed if necessary
	// uid    - userid that will passed to the Apache Derby database
	// pwd    - password that will be passed to the database
	//  
    public void connectYMLD(Object panel, String uid, String pwd) throws Exception {
		
		String DerbyDriver = "org.apache.derby.jdbc.EmbeddedDriver";
		
		try {
            Class.forName(DerbyDriver).newInstance();
		}
		catch (Exception NoDriver) {
            showMessage("Error: No suitable Apache Derby driver found.");
            return;
		}
		
		try {
            String url = "jdbc:derby:YMLD";			   // Connect to the YMLD database
            Properties properties = new Properties();  // Set the connection properties
            properties.put("user", uid); 			   // Send userid and password
            properties.put("password", pwd);
            conn = DriverManager.getConnection(url,properties);// Connect!
            Statement s = conn.createStatement();	   // Create a statement
            s.execute("SET SCHEMA APP");			   // Set the SCHEMA to APP
            remove(panel);							   // Remove the connect panel
		}
		catch(SQLException se) {					   // Trap all SQL errors
            getSQLError(se);
		}
		catch(Exception e) {						   // Driver error
            showMessage("Error: Cannot connect to JDBC Driver.");
		}
	}	
  
	//
	// pClosePanel() closes any panel that is currently displayed.
	// Any panel that has a close button will call this method with
	// pClosePanel(panel object).
	// panel    - an object reference to the panel. Set the 
	//            name field of the main panel to "panel" and the
	//            action field to pClosePanel(panel).
	//  
    public void pClosePanel(Object panel) {
		remove(panel);                                 // Get rid of the panel
	}
	
	//
	// exitYMLD() exists the YMLD program. This method also closes the connection
	// if it is still open.
	// 
	public void exitYMLD() throws Exception {
		try {
			if (conn != null) {conn.close();}	       // Close only if open!
		}
		catch (Exception se) {                         // Don't care about errors
		}			
		System.exit(0);                                // Exit completely
	}
    
    //
    // info() displays the "About" screen that describes the YMLD application
    //
	public void info() throws Exception {
		Object panel = parse(ymldInfo);                // Open up the info panel
		add(panel);
		requestFocus(find(panel,"butclose"));          // Focus on close button
	}    
    
	//
    // showMessage() displays an error message on the screen.
    // msg      - any text message to be displayed
    //
	public void showMessage(String msg) throws Exception {
		Object panel = parse(ymldMessage);             // Open the message screen
		add(panel);
		setString(find(panel,"msg"),"text",msg);       // Fill the message field
		requestFocus(find(panel,"bOK"));               // Focus on the OK button
	}
		
    // 
    // getSQLError() will display the SQL error messages.
    // Whenever an SQLException is produced by any of the methods, the
    // getSQLError routine is called. This routine will display the 
	// error message on the screen using the showMessage method.
	// se        - SQLException information.
    //
	public void getSQLError(SQLException se) throws Exception {
        String SQLState = se.getSQLState();            // Get the error state 
        String SQLMessage = se.getMessage();           // And message
		showMessage("SQLState("+SQLState+ "): " + SQLMessage); // Display
    }    
	
    //
    // Current productions Panel and Logic
    // pShowproductions() displays all of the productions currently showing at 
    // at the YMLD Theater
    //    
	public void pShowProductions() throws Exception {
		
		String columns [] = new String[5];             // 5 columns of information
		Object panel = parse(ymldProductions);         // Open the productions screen
		add(panel);
		
        try {
            Statement s = conn.createStatement();      // Connect to the database
            ResultSet rs = s.executeQuery( "SELECT * FROM Productions");
            while (rs.next()) {                        // Retrieve all productions
				Object row = create("row");            // Create a row in the display
				for (int i = 1; i < 5; ++i) {          // Get all five columns
					Object cell = create("cell");      // Create a cell for each column
					setString(cell,"text",rs.getString(i));
					add(row,cell);                     // Add row to display table
				}
                Object tablep = find(panel,"Prodtable");
                add(tablep,row);				
            }
            rs.close();                                // Close cursors 
            s.close();
        }
        catch (SQLException se) {                      // In case of an error!
			getSQLError(se);
		}
	}    
    
    //
    // Show Seat arrangement in the theater
    // pShowSeats() displays a map of the theater seats.
    //
	public void pShowSeats() throws Exception {
		Object panel = parse(ymldShowSeats);           // Display seat map
		add(panel);
	}    
    
    //
    // Show Seat price plans at the theater
    // pShowPricePlans() displays the current price plans that the YMLD
	// system has.
    //
	public void pShowPricePlans() throws Exception {
		Object panel = parse(ymldPricePlans);          // Open the pricing display
		add(panel);
			
		try {
			Statement s = conn.createStatement();      // Connect to the database
			ResultSet rs = s.executeQuery( "SELECT * FROM PRICEPLAN");
			while (rs.next()) {                        // For all price plans
				Object row = create("row");            // Display data on screen
				for (int  i = 1; i < 3; ++i) {
					Object cell = create("cell");      // Create cells for each column
					setString(cell,"text",rs.getString(i));
					add(row,cell);                     // Add row to table
				}
				Object tablep = find(panel,"PriceTable");
				add(tablep,row);				      
			}
			rs.close();                                // Close all cursors 
			s.close();
		}
		catch (SQLException se) {                      // In case of an error
			getSQLError(se);
		}			
	}
	     
    //
    // Undo Transaction Panels and Logic
    //
    // This section contains the code necessary to remove (refund) a transaction
    // that has taken place in the YMLD database. There are five routines
    // associated with undoing a transaction
    // pShowTransactions    - Shows all transactions in the YMLD database
    // selectRow            - Displays the confirmation message for undoing a transaction
    // undoTx               - Deletes the transaction in the database
    //
    
    //
    // pShowTransactions() displays all of the transactions in the YMLD database.
    // A user will select a transaction on the screen to refund.
    //
    public void pShowTransactions() throws Exception {
		Object panel = parse(ymldTransactions);
		add(panel);                                    // Display panel
		
        try {
            Statement s = conn.createStatement();      // Select all purchases
            ResultSet rs = s.executeQuery("SELECT * FROM TRANSACTIONS WHERE TX_TYPE='PURCHASE'");
			
			while (rs.next()) { 
                String txNo   = rs.getString("TX_NO"); // Retrieve the fields from table
				String txType = rs.getString("TX_TYPE");
				String txDate = rs.getString("TX_DATE");
				String perfNo = rs.getString("PERFORMANCE_NO");
				String seatNo = rs.getString("SEAT_NO");
				Object row = create("row");            // Place the data on the screen
                Object cTxNo   = create("cell"); setString(cTxNo,  "text",txNo);   add(row,cTxNo);
                Object cTxType = create("cell"); setString(cTxType,"text",txType); add(row,cTxType);
                Object cTxDate = create("cell"); setString(cTxDate,"text",txDate); add(row,cTxDate);
                Object cPerfNo = create("cell"); setString(cPerfNo,"text",perfNo); add(row,cPerfNo);
                Object cSeatNo = create("cell"); setString(cSeatNo,"text",seatNo); add(row,cSeatNo);
				Object tablep = find(panel,"txTable");
                add(tablep,row);								
            }
            rs.close(); 
            s.close();
        }
        catch (SQLException se) {                      // Hopefully no errors!
			getSQLError(se);
		}						
	}    
    
    //
    // selectRow() is invoked when a user selects a transaction on the screen.
    // The information on the transaction is retrieved and presented in the 
    // ymldUndo panel for confirmation.
    //
    public void selectRow(Object txTable) throws Exception {
		currentRow = getSelectedItem(txTable);         // Get the selected transaction
		Object pdeleteTxNo = getItem(currentRow,0);    // Get all of the fields
        Object pdeleteDate = getItem(currentRow,2);
		Object pdeletePerfNo = getItem(currentRow,3);
		Object pdeleteSeatNo = getItem(currentRow,4);
		deleteDate   = getString(pdeleteDate,"text");  // Set all of the variables for delete
		deletePerfNo = getString(pdeletePerfNo,"text");
		deleteSeatNo = getString(pdeleteSeatNo,"text");
		deleteTxNo   = getString(pdeleteTxNo,"text");
		Object panel = parse(ymldUndo);                // Load the UNDO panle
		add(panel);                        
		Object txtarea = find(panel,"msg");            // Find the message object
		setString(txtarea,"text","Do you want to undo the reservation for performance " + 
		      deletePerfNo + ", seat " + deleteSeatNo + " on " + deleteDate + "?");
		requestFocus(find(panel,"bCancel"));           // Focus on the Cancel button
	}
    
    //
    // undoTx() is called to delete (undo) the purchase transaction.
    // There are 3 steps associated with undoing a transaction:
    // 1) The seat must be released back to the performance for resale
    // 2) The transaction must be voided
    // 3) The transaction log must contain a record of the refund.
    //
	public void undoTx(Object panel) throws Exception {
		try {
			conn.setAutoCommit(false);                 // Need to commit all TXs
			PreparedStatement psUpdate = conn.prepareStatement( // Free up seat
			  "UPDATE SEATS SET SEAT_AVAILABLE = 1" +
				"WHERE SEAT_NO = ? AND PERFORMANCE_NO = ?") ;
            psUpdate.setInt(1,Integer.parseInt(deleteSeatNo));
            psUpdate.setInt(2,Integer.parseInt(deletePerfNo));
            int rows = psUpdate.executeUpdate();
			PreparedStatement psInsert = conn.prepareStatement( // Refund record
			  "INSERT INTO TRANSACTIONS(Tx_Type,Tx_Date,Performance_No,Seat_No) " +
				" VALUES ('REFUND',CURRENT_DATE, ?, ?)");
            psInsert.setInt(1,Integer.parseInt(deletePerfNo));
            psInsert.setInt(2,Integer.parseInt(deleteSeatNo));
			rows = psInsert.executeUpdate();
			PreparedStatement psUpdate2 = conn.prepareStatement( // Void old record
			  "UPDATE TRANSACTIONS SET TX_TYPE = 'VOID' " +
				"WHERE TX_NO = ?");
            psUpdate2.setInt(1,Integer.parseInt(deleteTxNo));
			rows = psUpdate2.executeUpdate();	
			conn.commit();                             // Commit the transaction
			remove(currentRow);                        // Remove current row in display
			psInsert.close();                          // Close all cursors
			psUpdate.close();
			psUpdate2.close();
			conn.setAutoCommit(true);			       // Turn autocommit back on
		}
        catch (SQLException se) {                      // Just in case!
			getSQLError(se);
		}		
		remove(panel);                                 // Remove panel when done
	}
	    
	//
    // Seat Selection Logic
    //
    // The seat selection logic consists of the following methods and panels:
    // pProdSelection()     - Select a production to purchase a ticket on
    // showSeatSelection()  - Show the seats for a production
    // bCalendar()          - Show available seats for a performance
    // pShowTickets()       - Show the tickets that were selected
    // pFinishPurchase(0    - Finalize the purchase of the tickets
    // pNextMonth()         - Move the calendar forward for a production
    // pPrevMonth()         - Move the calendar backward for a production
    // setCalendar()        - Set the dates on the panels with the production dates
    // bSelectProd()        - Logic when a production is selected
    // bSelectSeat()        - Logic when a seat is selected
    //
    
	//
    // pProdSelection() displays the available productions as a menu. The 
    // user selects a production to buy tickets from and then control is 
    // passed onto the seat selection screen.
    //
	public void pProdSelection() throws Exception {
		
		panelProdSelection = parse(ymldProdSelection); // Open the production screen
        try {
            Statement s = conn.createStatement();      // Connect to the database
            ResultSet rs = s.executeQuery(             // Select the productions
			  "SELECT PRODUCTION_NO, PRODUCTION_TITLE, " +
				"       MONTH(PRODUCTION_START) AS MONTH_START, " + 
				"       MONTH(PRODUCTION_END) AS MONTH_END " +
				"FROM PRODUCTIONS " +
				"ORDER BY PRODUCTION_NO");
			Object buttons = find(panelProdSelection,"buttons");
			int butpos = 0;                            // Set all of the buttons
            while (rs.next()) {                        // processing the rows 
                int production_no = rs.getInt("PRODUCTION_NO"); // Get fields
                String production_title = rs.getString("PRODUCTION_TITLE");
				String production_start = rs.getString("MONTH_START");
				String production_end = rs.getString("MONTH_END");
				productionNames[production_no] = production_title;
				Object button = getItem(buttons,(butpos*2));
				putProperty(button,"no",String.valueOf(production_no));
				putProperty(button,"start",production_start);
				putProperty(button,"end",production_end);
				Object label = getItem(buttons,(butpos*2)+1);
				setString(label,"text",production_title);
				butpos += 1;                           // Only room for 9 buttons
				if (butpos == 9) break;
            }
			add(panelProdSelection);                   // Display the screen   
            rs.close();                                // Close the cursors 
            s.close();
        }
        catch (SQLException se) {                      // In case of an error
			getSQLError(se);
		}
	}    
    
    //
    // bSelectProd() is called when one of the productions has been selected
    // from the productions panel. The button object is used to determine which
    // production was selected.
    //
	public void bSelectProd(Object button) throws Exception {
        Object butno = getProperty(button,"no");       // Get the button #
		selectedProdNo = Integer.parseInt(String.valueOf(butno)); // Thats the prodiction
		Object start = getProperty(button,"start");    // Get the start/end dates of the production
		minMonth = Integer.parseInt(String.valueOf(start));
		Object end = getProperty(button,"end");
		maxMonth = Integer.parseInt(String.valueOf(end));
		selectedProdMonth = minMonth;                  // Set the starting month to display
		selectedProdName = productionNames[selectedProdNo]; // Title of the production
		showSeatSelection();                           // Show the seat selection screen
	} 

    //
    // showSeatSelection() displays the seat select screen. It also calls the 
    // setCalendar() method to set the calendar to the proper date and days that
    // a specific production is available on.
    //
	public void showSeatSelection() throws Exception {
		panelSeatSelection = parse(ymldSeatSelection); // Show the seat selection screen
		add(panelSeatSelection);
		Object title = find(panelSeatSelection,"production"); // Set the title on the screen
		setString(title,"text","Production: "+productionNames[selectedProdNo]);
		selectedProdMonth = minMonth;                  // Set th starting month to show
		setCalendar();                                 // Set the calendar display
	}
	
    //
    // setCalendar() updates the Seat display to show the current month, and the days
    // that a performance is available. This method relies on the getProdDates() method
    // to find what dates during a month that a production is available.
    //
	public void setCalendar() throws Exception {
		
		getProdDates(selectedProdNo, selectedProdMonth);// Get current production dates
		Object objMonth = find(panelSeatSelection,"month");	// Get panel with month information on it
		setString(objMonth,"text",monthNames[selectedProdMonth]);// Set text of current month		

		Object daysPanel = find(panelSeatSelection,"days");	// Get days in calendar
		for (int i = 0; i <= 41; ++i) {  			   // Clear all calendar entries
			Object dayButton = getItem(daysPanel,i);
			setString(dayButton,"text","");            // Zap date string
			setBoolean(dayButton,"enabled",false);
		}
		// Now print the days in each of the buttons and make them visible
		int dayofmonth = 1;
		for (int i = monthDoW[selectedProdMonth]; i < monthDoW[selectedProdMonth]+monthDays[selectedProdMonth]; ++i) {
			Object dayButton = getItem(daysPanel,i);
			setString(dayButton,"text",String.valueOf(dayofmonth));
			if (prodDays[dayofmonth] == true) {        // If there is a performances on a date, set it to visible
				setBoolean(dayButton,"enabled",true);
			}
			dayofmonth++;
		}
		
		for (int i = 1; i <= 20; ++i) {                // Reset all of the seats to be blank
			Object seat = find(panelSeatSelection,String.valueOf(i));
			setIcon(seat,"icon",null);
			putProperty(seat,"selected","0");
			setBoolean(seat,"enabled",false);
		}
		
		totalCost = 0;                                 // Zero tickets purchased so far
	}
    
	//
    // getProdDates() is responsible for figuring out what dates a production
    // is on during a particular month. This information is used by the 
    // calendar routine to highlight the dates on the calendar where a user
    // can purchase a ticket.
    //
	public void getProdDates(int prodNo, int prodMonth) throws Exception {
		
        try {
            Statement s = conn.createStatement();      // Create a statement 
            ResultSet rs = s.executeQuery(             // Find all dates in a month
			      "SELECT DAY(PERFORMANCE_DATE) AS DAYMONTH, PERFORMANCE_NO " + 
                  "FROM PERFORMANCES P " +
                  "WHERE P.PRODUCTION_NO = " + prodNo + " AND " +
                  "      MONTH(P.PERFORMANCE_DATE) = " + prodMonth + " AND " +
                  "      P.PERFORMANCE_SEATS > 0 " +
                  "ORDER BY PRODUCTION_NO");
			for (int i = 1; i <= 31; ++i) prodDays[i] = false; // Set all dates false
			while (rs.next()) {                        // Based on the result set, set dates
                int dayOfMonth = rs.getInt(1);         // Day of the month (for a performance)
				int perfNo = rs.getInt(2);             // The performance number for that date
				prodDays[dayOfMonth] = true;           // Set the day value to true
				prodPerf[dayOfMonth] = perfNo;         // And keep the performance number 
            }
            rs.close();                                // Close the statements and cursors
            s.close();
        }
        catch (SQLException se) {                      // Just in case something went wrong
			getSQLError(se);
		}			
	}    

    //
    // pNextMonth() and pPrevMonth() are used to change the month on the screen. If
    // the user goes beyond the end of a production (or before), the current month
    // stays the same.
    //
	public void pNextMonth() throws Exception {
		selectedProdMonth++;                           // Increment the month
		if (selectedProdMonth > maxMonth) selectedProdMonth = maxMonth;
        setCalendar();                                 // Set the calendar
	}
	
	public void pPrevMonth() throws Exception {
		selectedProdMonth--;                           // Decrease the month
		if (selectedProdMonth < minMonth) selectedProdMonth = minMonth;
		setCalendar();                                 // Set the calendar
	}
	       
    //
    // bCalendar() is called when someone selects a date on the screen. When 
    // a date is selected, the seats are updated to show which ones are 
    // available for purchase.
    //
	public void bCalendar(Object objButton) throws Exception {
        String whichDay = getString(objButton,"text"); // Figure out which date was picked
		selectedProdDay = Integer.parseInt(whichDay);  // Which day was selected
		selectedProdDate = "2005-" + selectedProdMonth + "-" + selectedProdDay;
        selectedPerfNo = prodPerf[selectedProdDay];    // Current performance number
					
        try {
            Statement s = conn.createStatement();      // Create a statement 
            ResultSet rs = s.executeQuery(             // Find available seats 
			  "SELECT SEAT_NO, SEAT_AVAILABLE, PRICEPLAN_COST " + 
				"FROM SEATS S, PRICEPLAN P " +
				"WHERE PERFORMANCE_NO = " + selectedPerfNo + " AND " +
				"      S.PRICEPLAN_NO = P.PRICEPLAN_NO " +
				"ORDER BY SEAT_NO");
			
            for (int i = 1; i <= 20; ++i) {            // Set all seats to null for now
                Object seat = find(panelSeatSelection,String.valueOf(i));
                setIcon(seat,"icon",null);
                setBoolean(seat,"enabled",false);
            }    
        
            while (rs.next()) {                        // While we have seats 
                int seatNo = rs.getInt(1);             // Get the seat number 
                int seatAvail = rs.getInt(2);          // Is it available?
                int seatPrice = rs.getInt(3);          // What's the price?
                Object seat = find(panelSeatSelection,String.valueOf(seatNo));
                if (seatAvail == 0) {                  // The seat already sold
                    setIcon(seat,"icon",getIcon("panels/I-Small-Cancel.png"));
                    setBoolean(seat,"enabled",false);
                }
                else {                                 // The seat is available
                    setBoolean(seat,"enabled",true);
                    putProperty(seat,"selected","0");
                    putProperty(seat,"price",String.valueOf(seatPrice));
                }
            }
            rs.close();                                // Close statements 
            s.close();
        }                                              
        catch (SQLException se) {                      // If errors
            getSQLError(se);
        }				
    }
	
    //
    // bSeat() is called when someone selects a seat for purchase.
    //
	public void bSeat(Object seat) throws Exception {  
		Object seatSelected = getProperty(seat,"selected"); // Make seat selected
		Object seatPrice = getProperty(seat,"price");		
		Object showPrice = find(panelSeatSelection,"price");
		if (String.valueOf(seatSelected) == "1") {     // Toggle between selected and not
			putProperty(seat,"selected","0");          // Turn back to blank 
			setIcon(seat,"icon",null);
			totalCost -= Integer.parseInt(String.valueOf(seatPrice)); // Subtract from price
		}
		else {                                         // Seat now selected
			putProperty(seat,"selected","1");          // Change to green checked
			setIcon(seat,"icon",getIcon("panels/I-OK.png")); // Add price of seat to total
			totalCost += Integer.parseInt(String.valueOf(seatPrice));			
		}
        setString(showPrice,"text","$"+String.valueOf(totalCost));
	}

    //
    // pShowTickets() list all of the tickets that the user is going to purchase
    // for a performance. It places the information into a table that the user
    // would check, and then finalize the purchase.
    //
	public void pShowTickets(Object Inprice) throws Exception {
		if (totalCost == 0) {                          // Zero price means nothing selected
			showMessage("No seats selected!");
			return;
		}
		panelConfirmSelection = parse(ymldShowTickets); // Final ticket list display
		add(panelConfirmSelection);                   
		Object prodno = find(panelConfirmSelection,"prodno"); // Show production information
		setString(prodno,"text",String.valueOf(selectedProdNo));
		Object prodtitle = find(panelConfirmSelection,"prodtitle");
		setString(prodtitle,"text",selectedProdName);
		Object proddate = find(panelConfirmSelection,"date");
		setString(proddate,"text",selectedProdDate);
				
		for (int i = 1; i <= 20; ++i) {                // Check each seat to see which are selected
			Object button = find(panelSeatSelection,String.valueOf(i));
			if (getBoolean(button,"enabled") == false) continue; // Not selected so ignore
			Object selected = getProperty(button,"selected"); // Get selection price
			Object price = getProperty(button,"price");
			if (String.valueOf(selected) == "1") {     // If it is selected, place in list
				Object row = create("row");            // Row contains Seat # and price
				Object cell = create("cell");          // Create the cell for the column 
				setString(cell,"text",String.valueOf(i));	
				add(row,cell);
				cell = create("cell");
                setString(cell,"text",String.valueOf(price)); 
				add(row,cell);
				Object tablep = find(panelConfirmSelection,"tickets");
				add(tablep,row);				
			}
		}	
	}		

    //
    // pFinishPurchase() is called when the user has decided to finalize their
    // purchase. A list of seats that have been selected are placed into a table, and the
    // 
	public void pFinishPurchase() throws Exception {
        try {
			PreparedStatement psUpdate = conn.prepareStatement( // Update seat to sold
			  "UPDATE SEATS " + 
				"  SET SEAT_AVAILABLE = 0 " +
				"WHERE  SEAT_NO = ? AND PERFORMANCE_NO = ?") ;
  		PreparedStatement psLog = conn.prepareStatement(   // Update transaction log
			  "INSERT INTO TRANSACTIONS(Tx_Type,Tx_Date,Performance_No,Seat_No) " +
				" VALUES ('PURCHASE',CURRENT_DATE, ?, ?)");
		  for (int i = 1; i <= 20; ++i) {              // Check all seats
			  Object button = find(panelSeatSelection,String.valueOf(i));
			  if (getBoolean(button,"enabled") == false) continue;
			  Object selected = getProperty(button,"selected");
			  if (String.valueOf(selected) == "1") {   // All selected seats
					 psUpdate.setInt(1,i);             // Set the seat number
					 psUpdate.setInt(2,selectedPerfNo);// Set the performance #
					 int rows = psUpdate.executeUpdate(); // Update to sold
					 psLog.setInt(1,selectedPerfNo);   // Set the performance #
					 psLog.setInt(2,i);                // Set the seat number
					 rows = psLog.executeUpdate();     // Update the transaction log
				}
			}
			psUpdate.close();                          // Close the cursors
			psLog.close();
            remove(panelConfirmSelection);             // Remove all the panels on the screen
			remove(panelSeatSelection);
			remove(panelProdSelection);
			showMessage("Seat purchase complete.");    // Final purchase message
		}				
        catch (SQLException se) {                      // Just in case the SQL dies
			getSQLError(se);
		}		
	}
	
}
